#!/bin/sh
# Script to find MAC address of Omnia 11 IO board
# This script and it will edit dhcpd.conf so that the MAC address of that IO board is the only
# one recognized by dhcpd.
# USAGE:
#   'getmac'       : Edit dhcpd.conf with MAC from dhcpd.leases
#   'getmac reset' : Reset dhcpd.conf
#
# WARNINGS:
#   - IO board should be only DHCP client (we use last one in dhcpd.leases)

DHCPD_LEASES="/var/state/dhcp/dhcpd.leases"
DHCPD_CONF="/var/etc/dhcpd.proto.conf"
MACREGEX="..:..:..:..:..:.."

if [ "$1" == "reset" ]
then
	ALLOW_UNKNOWN="true"
	MAC="00:00:00:00:00:00"
	echo "Resetting dhcpd.conf..."
else
	# Get most recent hardware address from dhcpd.leases
	ALLOW_UNKNOWN="false"
	MAC=$(grep 'hardware' < $DHCPD_LEASES | tail -n 1 | sed 's|^.*\('"$MACREGEX"'\).*$|\1|')
	if [ $MAC ]
	then
		echo "Found MAC address: $MAC"
	else
		echo "Error: No MAC address found in $DHCPD_LEASES"
		exit 1
	fi
fi

# Edit dhcpd.conf
# Set boot-unknown-clients 
sed -i 's|^[^#]*boot-unknown-clients.*$|boot-unknown-clients '"$ALLOW_UNKNOWN"';|' $DHCPD_CONF

# Replace MAC address between 'o11io' and '}'
sed -i '/o11io/,/}/ s|'"$MACREGEX"'|'"$MAC"'|' $DHCPD_CONF

